import os
from datetime import datetime
from typing import Coroutine, overload, Literal

from ..core.param import RequestType
from ..core.interaction import Interaction
from ..solana_fm.client import SolanaFMClient, SolanaFMAsyncClient
from ..solana_fm.param import SolanaFMBlocksPaginationType
from ..solana_fm.schema import (
    GetAccountTransactionsParam,
    GetAccountTransactionsResponse,
    GetAccountTransfersParam,
    GetAccountTransfersResponse,
    GetAccountTransfersCsvExportParam,
    GetAccountTransfersCsvExportResponse,
    GetAccountTransactionsFeesResponse,
    GetBlocksResponse,
    GetBlockResponse,
    PostMultipleBlocksResponse,
    GetSolanaDailyTransactionFeesResponse,
    GetTaggedTokensListResponse,
    GetTokenInfoV0Response,
    PostTokenMultipleInfoV0Response,
    GetTokenInfoV1Response,
    PostTokenMultipleInfoV1Response,
    PostUserTokenAccountsResponse,
    GetMintTokenAccountsResponse,
    GetOnChainTokenDataResponse,
    GetTokenSupplyResponse,
    GetTransferTransactionsResponse,
    PostMultipleTransferTransactionsResponse,
    GetAllTransferActionsResponse
)


class SolanaFM(Interaction):
    """
        Class used to connect [SolanaFM](https://solana.fm/) API, popular site to interact and explore Solana blockchain.
        To have access SolanaFM API is **not** required an API key, but it is recommended to have one to increase the rate limit. 

        Check [https://docs.solana.fm/](https://docs.solana.fm/) for all the details on the available endpoints.

        !!! info
            If the API key is not provided during the object creation, then it is tried to be 
            retrieved also from ENV variable **SOLANA_FM_API_KEY**.

        Parameters:
            api_key: specify the API key to use for the connection.

        **Example**
        ```python
        import asyncio
        from cyhole.solana_fm import SolanaFM

        account = "ACCOUNT_ID"
        solana_fm = SolanaFM()

        # Get account transactions
        # synchronous
        response = solana_fm.client.get_account_transactions(account)
        print("Transactions Extracted:", len(response.result.data))

        # asynchronous
        async def main() -> None:
            async with solana_fm.async_client as client:
                response = await client.get_account_transactions(account)
                print("Transactions Extracted:", len(response.result.data))

        asyncio.run(main())
        ```
    """
    def __init__(self, api_key: str | None = None) -> None:

        # set API
        self.api_key = api_key
        if api_key is None:
            self.api_key = os.environ.get("SOLANA_FM_API_KEY")

        # headers setup
        headers = None
        if self.api_key is not None:
            headers = {
                "ApiKey": self.api_key
            }
        super().__init__(headers)

        # clients
        self.client = SolanaFMClient(self, headers = headers)
        self.async_client = SolanaFMAsyncClient(self, headers = headers)

        # API urls
        self.base_v0_url = "https://api.solana.fm/v0/"
        self.base_v1_url = "https://api.solana.fm/v1/"

        # private attributes
        self._name = "SolanaFM"
        self._description = "Interact with SolanaFM API"
        return

    def __str__(self) -> str:
        return self._name

    @overload
    def _get_account_transactions(self, sync: Literal[True], account: str, params: GetAccountTransactionsParam = GetAccountTransactionsParam()) -> GetAccountTransactionsResponse: ...

    @overload
    def _get_account_transactions(self, sync: Literal[False], account: str, params: GetAccountTransactionsParam = GetAccountTransactionsParam()) -> Coroutine[None, None, GetAccountTransactionsResponse]: ...

    def _get_account_transactions(self, sync: bool, account: str, params: GetAccountTransactionsParam = GetAccountTransactionsParam()) -> GetAccountTransactionsResponse | Coroutine[None, None, GetAccountTransactionsResponse]:
        """
            This function refers to the GET **[Account Transactions](https://docs.solana.fm/reference/get_account_transactions)** API endpoint, 
            and it is used to get the list of transactions for a given account according to input parameters.

            Parameters:
                account: The account address.
                params: The parameters to be used in the request.
                    More details in the object definition.

            Returns:
                List of transactions.
        """
        # set params
        url = self.base_v0_url + f"accounts/{account}/transactions"
        api_params = params.model_dump(
            by_alias = True,
            exclude_defaults = True
        )

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetAccountTransactionsResponse,
            params = api_params
        )

    @overload
    def _get_account_transactions_fees(self, sync: Literal[True], account: str, dt_from: datetime | None = None, dt_to: datetime | None = None) -> GetAccountTransactionsFeesResponse: ...

    @overload
    def _get_account_transactions_fees(self, sync: Literal[False], account: str, dt_from: datetime | None = None, dt_to: datetime | None = None) -> Coroutine[None, None, GetAccountTransactionsFeesResponse]: ...

    def _get_account_transactions_fees(self, sync: bool, account: str, dt_from: datetime | None = None, dt_to: datetime | None = None) -> GetAccountTransactionsFeesResponse | Coroutine[None, None, GetAccountTransactionsFeesResponse]:
        """
            This function refers to the GET **[Account Transactions Fees](https://docs.solana.fm/reference/get_account_tx_fees)** API endpoint,
            and it is used to get the list of transactions fees for a given account according to input parameters.

            Parameters:
                account: The account address.
                dt_from: The start date to filter transactions.
                dt_to: The end date to filter transactions.

            Returns:
                List of transactions fees.
        """
        # set params
        url = self.base_v0_url + f"accounts/{account}/fees"

        api_params = {}
        if dt_from is not None:
            api_params["from"] = dt_from.strftime("%Y-%m-%d")
        if dt_to is not None:
            api_params["to"] = dt_to.strftime("%Y-%m-%d")

        # execute request
        if sync:
            content_raw = self.client.api(RequestType.GET.value, url, params = api_params)
            return GetAccountTransactionsFeesResponse(data = content_raw.json())
        else:
            async def async_request():
                content_raw = await self.async_client.api(RequestType.GET.value, url, params = api_params)
                return GetAccountTransactionsFeesResponse(data = content_raw.json())
            return async_request()

    @overload
    def _get_account_transfers(self, sync: Literal[True], account: str, params: GetAccountTransfersParam = GetAccountTransfersParam()) -> GetAccountTransfersResponse: ...

    @overload
    def _get_account_transfers(self, sync: Literal[False], account: str, params: GetAccountTransfersParam = GetAccountTransfersParam()) -> Coroutine[None, None, GetAccountTransfersResponse]: ...

    def _get_account_transfers(self, sync: bool, account: str, params: GetAccountTransfersParam = GetAccountTransfersParam()) -> GetAccountTransfersResponse | Coroutine[None, None, GetAccountTransfersResponse]:
        """
            This function refers to the GET **[Account Transfers](https://docs.solana.fm/reference/get_account_transfers_v1)** API endpoint, 
            and it is used to get the list of transfers for a given account according to input parameters.

            Parameters:
                account: The account address.
                params: The parameters to be used in the request.
                    More details in the object definition.

            Returns:
                List of transfers.
        """
        # set params
        url = self.base_v0_url + f"accounts/{account}/transfers"
        api_params = params.model_dump(
            by_alias = True,
            exclude_defaults = True
        )

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetAccountTransfersResponse,
            params = api_params
        )

    @overload
    def _get_account_transfers_csv_export(self, sync: Literal[True], account: str, params: GetAccountTransfersCsvExportParam = GetAccountTransfersCsvExportParam()) -> GetAccountTransfersCsvExportResponse: ...

    @overload
    def _get_account_transfers_csv_export(self, sync: Literal[False], account: str, params: GetAccountTransfersCsvExportParam = GetAccountTransfersCsvExportParam()) -> Coroutine[None, None, GetAccountTransfersCsvExportResponse]: ...

    def _get_account_transfers_csv_export(self, sync: bool, account: str, params: GetAccountTransfersCsvExportParam = GetAccountTransfersCsvExportParam()) -> GetAccountTransfersCsvExportResponse | Coroutine[None, None, GetAccountTransfersCsvExportResponse]:
        """
            This function refers to the GET **[Account Transfers CSV Export](https://docs.solana.fm/reference/download_csv_v1)** API endpoint,
            and it is used to get the list of transfers for a given account according to input parameters in CSV format.

            Parameters:
                account: The account address.
                params: The parameters to be used in the request.
                    More details in the object definition.

            Returns:
                List of transfers in CSV format.
        """
        # set params
        url = self.base_v0_url + f"accounts/{account}/transfers/csv"
        api_params = params.model_dump(
            by_alias = True,
            exclude_defaults = True
        )

        # execute request
        if sync:
            content_raw = self.client.api(RequestType.GET.value, url, params = api_params)
            return GetAccountTransfersCsvExportResponse(csv = content_raw.text)
        else:
            async def async_request():
                content_raw = await self.async_client.api(RequestType.GET.value, url, params = api_params)
                return GetAccountTransfersCsvExportResponse(csv = content_raw.text)
            return async_request()

    @overload
    def _get_blocks(
            self,
            sync: Literal[True],
            from_block: int | None = None,
            page_size: int = 50,
            page_type: str = SolanaFMBlocksPaginationType.BLOCK_NUMBER.value,
            ascending: bool | None = None
        ) -> GetBlocksResponse: ...

    @overload
    def _get_blocks(
            self,
            sync: Literal[False],
            from_block: int | None = None,
            page_size: int = 50,
            page_type: str = SolanaFMBlocksPaginationType.BLOCK_NUMBER.value,
            ascending: bool | None = None
        ) -> Coroutine[None, None, GetBlocksResponse]: ...

    def _get_blocks(
        self,
        sync: bool,
        from_block: int | None = None,
        page_size: int = 50,
        page_type: str = SolanaFMBlocksPaginationType.BLOCK_NUMBER.value,
        ascending: bool | None = None
    ) -> GetBlocksResponse | Coroutine[None, None, GetBlocksResponse]:
        """
            This function refers to the GET **[Blocks](https://docs.solana.fm/reference/get_blocks_by_pagination)** API endpoint,
            and it is used to get the list of blocks according to input parameters.

            Parameters:
                from_block: The block number to start from.
                    If not provided, the latest block is returned.
                page_size: The number of blocks to return.
                page_type: The type of page to return.
                    The supported types are available on [`SolanaFMBlocksPaginationType`][cyhole.solana_fm.param.SolanaFMBlocksPaginationType].
                ascending: The order of the blocks.

            Returns:
                List of blocks.
        """
        # check param consistency
        SolanaFMBlocksPaginationType.check(page_type)

        # set params
        url = self.base_v0_url + "blocks"

        api_params = {
            "from": from_block,
            "pageSize": page_size,
            "paginationType": page_type,
            "reverse": ascending
        }

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetBlocksResponse,
            params = api_params
        )

    @overload
    def _get_block(self, sync: Literal[True], block_number: int) -> GetBlockResponse: ...

    @overload
    def _get_block(self, sync: Literal[False], block_number: int) -> Coroutine[None, None, GetBlockResponse]: ...

    def _get_block(self, sync: bool, block_number: int) -> GetBlockResponse | Coroutine[None, None, GetBlockResponse]:
        """
            This function refers to the GET **[Block](https://docs.solana.fm/reference/get_block)** API endpoint,
            and it is used to get the block details for a given block Number.

            Parameters:
                block_number: The block number.

            Returns:
                Block details.
        """
        # set params
        url = self.base_v0_url + f"blocks/{block_number}"

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetBlockResponse
        )

    @overload
    def _post_multiple_blocks(self, sync: Literal[True], block_numbers: list[int], producer_details: bool = True) -> PostMultipleBlocksResponse: ...

    @overload
    def _post_multiple_blocks(self, sync: Literal[False], block_numbers: list[int], producer_details: bool = True) -> Coroutine[None, None, PostMultipleBlocksResponse]: ...

    def _post_multiple_blocks(self, sync: bool, block_numbers: list[int], producer_details: bool = True) -> PostMultipleBlocksResponse | Coroutine[None, None, PostMultipleBlocksResponse]:
        """
            This function refers to the POST **[Multiple Blocks](https://docs.solana.fm/reference/get_multiple_blocks)** API endpoint,
            and it is used to get multiple blocks information from the SolanaFM API.

            Parameters:
                block_numbers: The list of block numbers to get.

            Returns:
                Block details.
        """
        # set params
        url = self.base_v0_url + "blocks"

        headers = {
            "accept": "application/json",
            "content-type": "application/json"
        }

        json = {
            "hydration": { "accountHash": producer_details },
            "blockNumbers": block_numbers,
        }

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.POST.value,
            url = url,
            response_model = PostMultipleBlocksResponse,
            headers = headers,
            json = json
        )

    @overload
    def _get_solana_daily_transaction_fees(self, sync: Literal[True], dt: datetime = datetime.now()) -> GetSolanaDailyTransactionFeesResponse: ...

    @overload
    def _get_solana_daily_transaction_fees(self, sync: Literal[False], dt: datetime = datetime.now()) -> Coroutine[None, None, GetSolanaDailyTransactionFeesResponse]: ...

    def _get_solana_daily_transaction_fees(self, sync: bool, dt: datetime = datetime.now()) -> GetSolanaDailyTransactionFeesResponse | Coroutine[None, None, GetSolanaDailyTransactionFeesResponse]:
        """
            This function refers to the GET **[Solana Daily Transaction Fees](https://docs.solana.fm/reference/get_daily_tx_fees)** API endpoint,
            and it is used to get the daily transaction fees for a given date on the whole Solana chain.

            Observe that if the date is not provided, the current date is used and the result could change over time 
            as the day is not finished yet.

            Parameters:
                dt: The date to get the transaction fees.
                    By default, the current date is used.

            Returns:
                Daily transaction fees.
        """
        # set params
        url = self.base_v0_url + "stats/tx-fees"

        api_params = {
            "date": dt.strftime("%d-%m-%Y")
        }

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetSolanaDailyTransactionFeesResponse,
            params = api_params
        )

    @overload
    def _get_tagged_tokens_list(self, sync: Literal[True]) -> GetTaggedTokensListResponse: ...

    @overload
    def _get_tagged_tokens_list(self, sync: Literal[False]) -> Coroutine[None, None, GetTaggedTokensListResponse]: ...

    def _get_tagged_tokens_list(self, sync: bool) -> GetTaggedTokensListResponse | Coroutine[None, None, GetTaggedTokensListResponse]:
        """
            This function refers to the GET **[Tagged Tokens List](https://docs.solana.fm/reference/get_tokens_by_pagination)** API endpoint,
            and it is used to get the list of tagged tokens identified by the SolanaFM team (**not tokens indexed on-chain**).

            Returns:
                List of tagged tokens.
        """
        # set params
        url = self.base_v0_url + "tokens"

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetTaggedTokensListResponse
        )

    @overload
    def _get_token_info_v0(self, sync: Literal[True], address: str) -> GetTokenInfoV0Response: ...

    @overload
    def _get_token_info_v0(self, sync: Literal[False], address: str) -> Coroutine[None, None, GetTokenInfoV0Response]: ...

    def _get_token_info_v0(self, sync: bool, address: str) -> GetTokenInfoV0Response | Coroutine[None, None, GetTokenInfoV0Response]:
        """
            This function refers to the GET **[Token Info V0](https://docs.solana.fm/reference/get_token_by_account_hash)** API endpoint,
            and it is used to get the token information for a given token address.

            !!! info
                This endpoint refers to the **V0** version of the API.

            Parameters:
                address: The token address.

            Returns:
                Token information.
        """
        # set params
        url = self.base_v0_url + f"tokens/{address}"

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetTokenInfoV0Response
        )

    @overload
    def _post_token_multiple_info_v0(self, sync: Literal[True], addresses: list[str]) -> PostTokenMultipleInfoV0Response: ...

    @overload
    def _post_token_multiple_info_v0(self, sync: Literal[False], addresses: list[str]) -> Coroutine[None, None, PostTokenMultipleInfoV0Response]: ...

    def _post_token_multiple_info_v0(self, sync: bool, addresses: list[str]) -> PostTokenMultipleInfoV0Response | Coroutine[None, None, PostTokenMultipleInfoV0Response]:
        """
            This function refers to the POST **[Token Multiple Info V0](https://docs.solana.fm/reference/get_tokens_by_account_hashes)** API endpoint,
            and it is used to get the token information for multiple token addresses.

            !!! info
                This endpoint refers to the **V0** version of the API.

            Parameters:
                addresses: The list of token addresses.

            Returns:
                Token information.
        """
        # set params
        url = self.base_v0_url + "tokens"

        headers = {
            "accept": "application/json",
            "content-type": "application/json"
        }

        json = {
            "hydration": { "accountHash": True },
            "tokenHashes": addresses
        }

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.POST.value,
            url = url,
            response_model = PostTokenMultipleInfoV0Response,
            headers = headers,
            json = json
        )

    @overload
    def _get_token_info_v1(self, sync: Literal[True], address: str) -> GetTokenInfoV1Response: ...

    @overload
    def _get_token_info_v1(self, sync: Literal[False], address: str) -> Coroutine[None, None, GetTokenInfoV1Response]: ...

    def _get_token_info_v1(self, sync: bool, address: str) -> GetTokenInfoV1Response | Coroutine[None, None, GetTokenInfoV1Response]:
        """
            This function refers to the GET **[Token Info V1](https://docs.solana.fm/reference/get_one_token)** API endpoint,
            and it is used to get the token information for a given token address.

            !!! info
                This endpoint refers to the **V1** version of the API.

            Parameters:
                address: The token address.

            Returns:
                Token information.
        """
        # set params
        url = self.base_v1_url + f"tokens/{address}"

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetTokenInfoV1Response
        )

    @overload
    def _post_token_multiple_info_v1(self, sync: Literal[True], addresses: list[str]) -> PostTokenMultipleInfoV1Response: ...

    @overload
    def _post_token_multiple_info_v1(self, sync: Literal[False], addresses: list[str]) -> Coroutine[None, None, PostTokenMultipleInfoV1Response]: ...

    def _post_token_multiple_info_v1(self, sync: bool, addresses: list[str]) -> PostTokenMultipleInfoV1Response | Coroutine[None, None, PostTokenMultipleInfoV1Response]:
        """
            This function refers to the POST **[Token Multiple Info V1](https://docs.solana.fm/reference/retrieve_multiple_tokens)** API endpoint,
            and it is used to get the token information for multiple token addresses.

            !!! info
                This endpoint refers to the **V1** version of the API.

            Parameters:
                addresses: The list of token addresses.

            Returns:
                Token information.
        """
        # set params
        url = self.base_v1_url + "tokens"

        headers = {
            "accept": "application/json",
            "content-type": "application/json"
        }

        json = {
            "tokens": addresses
        }

        # execute request
        if sync:
            content_raw = self.client.api(RequestType.POST.value, url, headers = headers, json = json)
            return PostTokenMultipleInfoV1Response(tokens = content_raw.json())
        else:
            async def async_request():
                content_raw = await self.async_client.api(RequestType.POST.value, url, headers = headers, json = json)
                return PostTokenMultipleInfoV1Response(tokens = content_raw.json())
            return async_request()

    @overload
    def _post_user_token_accounts(self, sync: Literal[True], address: str, include_sol_balance: bool = False, tokens: list[str] | None = None) -> PostUserTokenAccountsResponse: ...

    @overload
    def _post_user_token_accounts(self, sync: Literal[False], address: str, include_sol_balance: bool = False, tokens: list[str] | None = None) -> Coroutine[None, None, PostUserTokenAccountsResponse]: ...

    def _post_user_token_accounts(self, sync: bool, address: str, include_sol_balance: bool = False, tokens: list[str] | None = None) -> PostUserTokenAccountsResponse | Coroutine[None, None, PostUserTokenAccountsResponse]:
        """
            This function refers to the POST **[User Token Accounts](https://docs.solana.fm/reference/get_user_token_accounts)** API endpoint,
            and it is used to get the token accounts for a given user address.

            Parameters:
                address: The user address.
                include_sol_balance: Flag to include the Sol balance of the account.
                tokens: The list of token addresses to filter the accounts.

            Returns:
                Token accounts.
        """
        # set params
        url = self.base_v1_url + f"tokens/{address}/token-accounts"

        headers = {
            "accept": "application/json",
            "content-type": "application/json"
        }

        json = {
            "includeSolBalance": include_sol_balance,
            "tokenHashes": tokens
        }

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.POST.value,
            url = url,
            response_model = PostUserTokenAccountsResponse,
            headers = headers,
            json = json
        )

    @overload
    def _get_mint_token_accounts(self, sync: Literal[True], address: str, page: int | None = None, page_size: int | None = None) -> GetMintTokenAccountsResponse: ...

    @overload
    def _get_mint_token_accounts(self, sync: Literal[False], address: str, page: int | None = None, page_size: int | None = None) -> Coroutine[None, None, GetMintTokenAccountsResponse]: ...

    def _get_mint_token_accounts(self, sync: bool, address: str, page: int | None = None, page_size: int | None = None) -> GetMintTokenAccountsResponse | Coroutine[None, None, GetMintTokenAccountsResponse]:
        """
            This function refers to the GET **[Mint Token Accounts](https://docs.solana.fm/reference/get_token_accounts_for_token_mint)** API endpoint,
            and it is used to get the token accounts owned by a given token mint address.

            Parameters:
                address: The token address.
                page: The page number.
                page_size: The number of accounts to return per page.

            Returns:
                Token accounts.
        """
        # set params
        url = self.base_v1_url + f"tokens/{address}/holders"

        api_params = {
            "page": page,
            "pageSize": page_size
        }

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetMintTokenAccountsResponse,
            params = api_params
        )

    @overload
    def _get_on_chain_token_data(self, sync: Literal[True], address: str) -> GetOnChainTokenDataResponse: ...

    @overload
    def _get_on_chain_token_data(self, sync: Literal[False], address: str) -> Coroutine[None, None, GetOnChainTokenDataResponse]: ...

    def _get_on_chain_token_data(self, sync: bool, address: str) -> GetOnChainTokenDataResponse | Coroutine[None, None, GetOnChainTokenDataResponse]:
        """
            This function refers to the GET **[On-Chain Token Data](https://docs.solana.fm/reference/get_tfi_token_data)** API endpoint,
            and it is used to get the token data for a given token address stored on-chain.

            Parameters:
                address: The token address.

            Returns:
                Token data.
        """
        # set params
        url = self.base_v1_url + f"tokens/{address}/info"

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetOnChainTokenDataResponse
        )

    @overload
    def _get_token_supply(self, sync: Literal[True], address: str) -> GetTokenSupplyResponse: ...

    @overload
    def _get_token_supply(self, sync: Literal[False], address: str) -> Coroutine[None, None, GetTokenSupplyResponse]: ...

    def _get_token_supply(self, sync: bool, address: str) -> GetTokenSupplyResponse | Coroutine[None, None, GetTokenSupplyResponse]:
        """
            This function refers to the GET **[Token Supply](https://docs.solana.fm/reference/get_token_circulating_supply)** API endpoint,
            and it is used to get the token circulating supply for a given token address.

            Parameters:
                address: The token address.

            Returns:
                Token supply.
        """
        # set params
        url = self.base_v1_url + f"tokens/{address}/supply"

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetTokenSupplyResponse
        )

    @overload
    def _get_transfer_transactions(self, sync: Literal[True], transaction: str) -> GetTransferTransactionsResponse: ...

    @overload
    def _get_transfer_transactions(self, sync: Literal[False], transaction: str) -> Coroutine[None, None, GetTransferTransactionsResponse]: ...

    def _get_transfer_transactions(self, sync: bool, transaction: str) -> GetTransferTransactionsResponse | Coroutine[None, None, GetTransferTransactionsResponse]:
        """
            This function refers to the GET **[Transfer Transactions](https://docs.solana.fm/reference/get_transfers)** API endpoint,
            and it is used to get the list of transfer transactions for a given account according to input parameters.

            Parameters:
                transaction: The transaction hash.

            Returns:
                List of transfer transactions.
        """
        # set params
        url = self.base_v0_url + f"transfers/{transaction}"

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.GET.value,
            url = url,
            response_model = GetTransferTransactionsResponse
        )

    @overload
    def _post_multiple_transfer_transactions(self, sync: Literal[True], transactions: list[str]) -> PostMultipleTransferTransactionsResponse: ...

    @overload
    def _post_multiple_transfer_transactions(self, sync: Literal[False], transactions: list[str]) -> Coroutine[None, None, PostMultipleTransferTransactionsResponse]: ...

    def _post_multiple_transfer_transactions(self, sync: bool, transactions: list[str]) -> PostMultipleTransferTransactionsResponse | Coroutine[None, None, PostMultipleTransferTransactionsResponse]:
        """
            This function refers to the POST **[Multiple Transfer Transactions](https://docs.solana.fm/reference/post_transfers)** API endpoint,
            and it is used to get multiple transfer transactions information from the SolanaFM API.

            Parameters:
                transactions: The list of transaction hashes to get.

            Returns:
                Transfer transactions details.
        """
        # set params
        url = self.base_v0_url + "transfers"

        headers = {
            "accept": "application/json",
            "content-type": "application/json"
        }

        json = {
            "transactionHashes": transactions
        }

        # execute request
        return  self.api_return_model(
            sync = sync,
            type = RequestType.POST.value,
            url = url,
            response_model = PostMultipleTransferTransactionsResponse,
            headers = headers,
            json = json
        )

    @overload
    def _get_all_transfer_actions(self, sync: Literal[True]) -> GetAllTransferActionsResponse: ...

    @overload
    def _get_all_transfer_actions(self, sync: Literal[False]) -> Coroutine[None, None, GetAllTransferActionsResponse]: ...

    def _get_all_transfer_actions(self, sync: bool) -> GetAllTransferActionsResponse | Coroutine[None, None, GetAllTransferActionsResponse]:
        """
            This function refers to the GET **[All Transfer Actions](https://docs.solana.fm/reference/get_actions)** API endpoint,
            and it is used to get the list of all transfer actions on SolanaFM.

            Returns:
                List of all transfer actions.
        """
        # set params
        url = self.base_v1_url + "actions"

        # execute request
        if sync:
            content_raw = self.client.api(RequestType.GET.value, url)
            return GetAllTransferActionsResponse(actions = content_raw.json())
        else:
            async def async_request():
                content_raw = await self.async_client.api(RequestType.GET.value, url)
                return GetAllTransferActionsResponse(actions = content_raw.json())
            return async_request()


import pytest
from pathlib import Path
from datetime import datetime

from pytest_mock import MockerFixture

from cyhole.solana_fm import SolanaFM
from cyhole.solana_fm.schema import (
    GetAccountTransactionsResponse,
    GetAccountTransfersParam,
    GetAccountTransfersResponse,
    GetAccountTransfersCsvExportParam,
    GetAccountTransfersCsvExportResponse,
    GetAccountTransactionsFeesResponse,
    GetBlocksResponse,
    GetBlockResponse,
    PostMultipleBlocksResponse,
    GetSolanaDailyTransactionFeesResponse,
    GetTaggedTokensListResponse,
    GetTokenInfoV0Response,
    PostTokenMultipleInfoV0Response,
    GetTokenInfoV1Response,
    PostTokenMultipleInfoV1Response,
    PostUserTokenAccountsResponse,
    GetMintTokenAccountsResponse,
    GetOnChainTokenDataResponse,
    GetTokenSupplyResponse,
    GetTransferTransactionsResponse,
    PostMultipleTransferTransactionsResponse,
    GetAllTransferActionsResponse
)
from cyhole.core.address.solana import JUP, USDC, SOL

# load test config
from .config import load_config, MockerManager
config = load_config()

# create resources folder
mock_path = Path(config.mock_folder) / config.solana_fm.mock_folder
mock_path.mkdir(parents = True, exist_ok = True)

class TestSolanaFM:
    """
        Class grouping all unit tests.
    """
    solana_fm = SolanaFM()
    mocker = MockerManager(mock_path)

    def test_get_account_transactions_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Account Transactions" for synchronous logic.

            Mock Response File: get_account_transactions.json
        """

        # load mock response
        mock_file_name = "get_account_transactions"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAccountTransactionsResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)
            
        # execute request
        response = self.solana_fm.client.get_account_transactions(JUP)

        # actual test
        assert isinstance(response, GetAccountTransactionsResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_account_transactions_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Account Transactions" for asynchronous logic.

            Mock Response File: get_account_transactions.json
        """

        # load mock response
        mock_file_name = "get_account_transactions"
        if config.mock_response or config.jupiter.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAccountTransactionsResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)
            
        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_account_transactions(JUP)

        # actual test
        assert isinstance(response, GetAccountTransactionsResponse)

    def test_get_account_transactions_fees_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Account Transactions Fees" for synchronous logic.

            Mock Response File: get_account_transactions_fees.json
        """

        # load mock response
        mock_file_name = "get_account_transactions_fees"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAccountTransactionsFeesResponse)

            # response content to be adjusted
            content = self.mocker.adjust_content_json(str(mock_response.json()["data"]))
            mock_response._content = content

            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_account_transactions_fees(JUP, dt_from = datetime(2024, 8, 15), dt_to = datetime(2024, 8, 20))

        # actual test
        assert isinstance(response, GetAccountTransactionsFeesResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_account_transactions_fees_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Account Transactions Fees" for asynchronous logic.

            Mock Response File: get_account_transactions_fees.json
        """

        # load mock response
        mock_file_name = "get_account_transactions_fees"
        if config.mock_response or config.jupiter.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAccountTransactionsFeesResponse)

            # response content to be adjusted
            content = self.mocker.adjust_content_json(str(mock_response.json()["data"]))
            mock_response._content = content

            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)
            
        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_account_transactions_fees(JUP, dt_from = datetime(2024, 8, 15), dt_to = datetime(2024, 8, 20))

        # actual test
        assert isinstance(response, GetAccountTransactionsFeesResponse)

    def test_get_account_transfers_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Account Transfers" for synchronous logic.

            Mock Response File: get_account_transfers.json
        """

        # load mock response
        mock_file_name = "get_account_transfers"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAccountTransfersResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)
            
        # execute request
        account = "5Q544fKrFoe6tsEbD7S8EmxGTJYAKtTVhAW5Q5pge4j1"
        params = GetAccountTransfersParam(
            utc_from_unix_time = int(datetime.now().timestamp()) - 5, # 5 seconds ago
            utc_to_unix_time = int(datetime.now().timestamp())
        )
        response = self.solana_fm.client.get_account_transfers(account, params)

        # actual test
        assert isinstance(response, GetAccountTransfersResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_account_transfers_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Account Transfers" for asynchronous logic.

            Mock Response File: get_account_transfers.json
        """

        # load mock response
        mock_file_name = "get_account_transfers"
        if config.mock_response or config.jupiter.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAccountTransfersResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)
            
        # execute request
        account = "5Q544fKrFoe6tsEbD7S8EmxGTJYAKtTVhAW5Q5pge4j1"
        params = GetAccountTransfersParam(
            utc_from_unix_time = int(datetime.now().timestamp()) - 5, # 5 seconds ago
            utc_to_unix_time = int(datetime.now().timestamp())
        )
        async with self.solana_fm.async_client as client:
            response = await client.get_account_transfers(account, params)

        # actual test
        assert isinstance(response, GetAccountTransfersResponse)

    def test_get_account_transfers_csv_export_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Account Transfers CSV Export" for synchronous logic.

            Mock Response File: get_account_transfers_csv_export.json
        """

        # load mock response
        mock_file_name = "get_account_transfers_csv_export"
        if config.mock_response or config.jupiter.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAccountTransfersCsvExportResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        account = "5Q544fKrFoe6tsEbD7S8EmxGTJYAKtTVhAW5Q5pge4j1"
        params = GetAccountTransfersCsvExportParam(
            utc_from_unix_time = int(datetime.now().timestamp()) - 5, # 5 seconds ago
            utc_to_unix_time = int(datetime.now().timestamp())
        )
        response = self.solana_fm.client.get_account_transfers_csv_export(account, params)

        # actual test
        assert isinstance(response, GetAccountTransfersCsvExportResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_account_transfers_csv_export_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Account Transfers CSV Export" for asynchronous logic.

            Mock Response File: get_account_transfers_csv_export.json
        """

        # load mock response
        mock_file_name = "get_account_transfers_csv_export"
        if config.mock_response or config.jupiter.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAccountTransfersCsvExportResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        account = "5Q544fKrFoe6tsEbD7S8EmxGTJYAKtTVhAW5Q5pge4j1"
        params = GetAccountTransfersCsvExportParam(
            utc_from_unix_time = int(datetime.now().timestamp()) - 5, # 5 seconds ago
            utc_to_unix_time = int(datetime.now().timestamp())
        )
        async with self.solana_fm.async_client as client:
            response = await client.get_account_transfers_csv_export(account, params)

        # actual test
        assert isinstance(response, GetAccountTransfersCsvExportResponse)

    def test_get_blocks_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Blocks" for synchronous logic.

            Mock Response File: get_blocks.json
        """

        # load mock response
        mock_file_name = "get_blocks"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetBlocksResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_blocks(page_size = 1)

        # actual test
        assert isinstance(response, GetBlocksResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_blocks_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Blocks" for asynchronous logic.

            Mock Response File: get_blocks.json
        """

        # load mock response
        mock_file_name = "get_blocks"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetBlocksResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_blocks(page_size = 1)

        # actual test
        assert isinstance(response, GetBlocksResponse)

    def test_get_block_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Block" for synchronous logic.

            Mock Response File: get_block.json
        """

        # load mock response
        mock_file_name = "get_block"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetBlockResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_block(19941305)

        # actual test
        assert isinstance(response, GetBlockResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_block_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Block" for asynchronous logic.

            Mock Response File: get_block.json
        """

        # load mock response
        mock_file_name = "get_block"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetBlockResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_block(19941305)

        # actual test
        assert isinstance(response, GetBlockResponse)

    def test_post_multiple_blocks_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Multiple Blocks" for synchronous logic.

            Mock Response File: post_multiple_blocks.json
        """

        # load mock response
        mock_file_name = "post_multiple_blocks"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostMultipleBlocksResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.post_multiple_blocks([180000000])

        # actual test
        assert isinstance(response, PostMultipleBlocksResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_post_multiple_blocks_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Multiple Blocks" for asynchronous logic.

            Mock Response File: post_multiple_blocks.json
        """

        # load mock response
        mock_file_name = "post_multiple_blocks"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostMultipleBlocksResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.post_multiple_blocks([180000000])

        # actual test
        assert isinstance(response, PostMultipleBlocksResponse)

    def test_post_multiple_blocks_no_producer_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Multiple Blocks" for synchronous logic by setting
            producer's details to false.

            Mock Response File: post_multiple_blocks_no_producer.json
        """

        # load mock response
        mock_file_name = "post_multiple_blocks_no_producer"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostMultipleBlocksResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.post_multiple_blocks([200000000], producer_details = False)

        # actual test
        assert isinstance(response, PostMultipleBlocksResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_post_multiple_blocks_no_producer_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Multiple Blocks" for asynchronous logic by setting
            producer's details to false.

            Mock Response File: post_multiple_blocks_no_producer.json
        """

        # load mock response
        mock_file_name = "post_multiple_blocks_no_producer"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostMultipleBlocksResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.post_multiple_blocks([200000000], producer_details = False)

        # actual test
        assert isinstance(response, PostMultipleBlocksResponse)

    def test_get_solana_daily_transaction_fees_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Solana Daily Transaction Fees" for synchronous logic.

            Mock Response File: get_solana_daily_transaction_fees.json
        """

        # load mock response
        mock_file_name = "get_solana_daily_transaction_fees"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetSolanaDailyTransactionFeesResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_solana_daily_transaction_fees()

        # actual test
        assert isinstance(response, GetSolanaDailyTransactionFeesResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_solana_daily_transaction_fees_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Solana Daily Transaction Fees" for asynchronous logic.

            Mock Response File: get_solana_daily_transaction_fees.json
        """

        # load mock response
        mock_file_name = "get_solana_daily_transaction_fees"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetSolanaDailyTransactionFeesResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_solana_daily_transaction_fees()

        # actual test
        assert isinstance(response, GetSolanaDailyTransactionFeesResponse)

    def test_get_tagged_tokens_list_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Tagged Tokens List" for synchronous logic.

            Mock Response File: get_tagged_tokens_list.json
        """

        # load mock response
        mock_file_name = "get_tagged_tokens_list"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTaggedTokensListResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_tagged_tokens_list()

        # actual test
        assert isinstance(response, GetTaggedTokensListResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_tagged_tokens_list_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Tagged Tokens List" for asynchronous logic.

            Mock Response File: get_tagged_tokens_list.json
        """

        # load mock response
        mock_file_name = "get_tagged_tokens_list"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTaggedTokensListResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_tagged_tokens_list()

        # actual test
        assert isinstance(response, GetTaggedTokensListResponse)

    def test_get_token_info_v0_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Token Info V0" for synchronous logic.

            Mock Response File: get_token_info_v0.json
        """

        # load mock response
        mock_file_name = "get_token_info_v0"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTokenInfoV0Response)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_token_info_v0(USDC)

        # actual test
        assert isinstance(response, GetTokenInfoV0Response)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_token_info_v0_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Token Info V0" for asynchronous logic.

            Mock Response File: get_token_info_v0.json
        """

        # load mock response
        mock_file_name = "get_token_info_v0"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTokenInfoV0Response)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_token_info_v0(USDC)

        # actual test
        assert isinstance(response, GetTokenInfoV0Response)

    def test_post_token_multiple_info_v0_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Token Multiple Info V0" for synchronous logic.

            Mock Response File: post_token_multiple_info_v0.json
        """

        # load mock response
        mock_file_name = "post_token_multiple_info_v0"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostTokenMultipleInfoV0Response)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.post_token_multiple_info_v0([USDC, SOL])

        # actual test
        assert isinstance(response, PostTokenMultipleInfoV0Response)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_post_token_multiple_info_v0_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Token Multiple Info V0" for asynchronous logic.

            Mock Response File: post_token_multiple_info_v0.json
        """

        # load mock response
        mock_file_name = "post_token_multiple_info_v0"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostTokenMultipleInfoV0Response)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.post_token_multiple_info_v0([USDC, SOL])

        # actual test
        assert isinstance(response, PostTokenMultipleInfoV0Response)

    def test_get_token_info_v1_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Token Info V1" for synchronous logic.

            Mock Response File: get_token_info_v1.json
        """

        # load mock response
        mock_file_name = "get_token_info_v1"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTokenInfoV1Response)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_token_info_v1(JUP)

        # actual test
        assert isinstance(response, GetTokenInfoV1Response)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_token_info_v1_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Token Info V1" for asynchronous logic.

            Mock Response File: get_token_info_v1.json
        """

        # load mock response
        mock_file_name = "get_token_info_v1"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTokenInfoV1Response)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_token_info_v1(JUP)

        # actual test
        assert isinstance(response, GetTokenInfoV1Response)

    def test_post_token_multiple_info_v1_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Token Multiple Info V1" for synchronous logic.

            Mock Response File: post_token_multiple_info_v1.json
        """

        # load mock response
        mock_file_name = "post_token_multiple_info_v1"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostTokenMultipleInfoV1Response)

            # response content to be adjusted
            content = self.mocker.adjust_content_json(str(mock_response.json()["tokens"]))
            mock_response._content = content

            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.post_token_multiple_info_v1([JUP, USDC])

        # actual test
        assert isinstance(response, PostTokenMultipleInfoV1Response)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_post_token_multiple_info_v1_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Token Multiple Info V1" for asynchronous logic.

            Mock Response File: post_token_multiple_info_v1.json
        """

        # load mock response
        mock_file_name = "post_token_multiple_info_v1"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostTokenMultipleInfoV1Response)

            # response content to be adjusted
            content = self.mocker.adjust_content_json(str(mock_response.json()["tokens"]))
            mock_response._content = content

            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.post_token_multiple_info_v1([JUP, USDC])

        # actual test
        assert isinstance(response, PostTokenMultipleInfoV1Response)

    def test_post_user_token_accounts_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post User Token Accounts" for synchronous logic.

            Mock Response File: post_user_token_accounts.json
        """

        # load mock response
        mock_file_name = "post_user_token_accounts"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostUserTokenAccountsResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.post_user_token_accounts(JUP)

        # actual test
        assert isinstance(response, PostUserTokenAccountsResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_post_user_token_accounts_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post User Token Accounts" for asynchronous logic.

            Mock Response File: post_user_token_accounts.json
        """

        # load mock response
        mock_file_name = "post_user_token_accounts"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostUserTokenAccountsResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.post_user_token_accounts(JUP)

        # actual test
        assert isinstance(response, PostUserTokenAccountsResponse)

    def test_get_mint_token_accounts_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Mint Token Accounts" for synchronous logic.

            Mock Response File: get_mint_token_accounts.json
        """

        # load mock response
        mock_file_name = "get_mint_token_accounts"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetMintTokenAccountsResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_mint_token_accounts(JUP, page_size = 2)

        # actual test
        assert isinstance(response, GetMintTokenAccountsResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_mint_token_accounts_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Mint Token Accounts" for asynchronous logic.

            Mock Response File: get_mint_token_accounts.json
        """

        # load mock response
        mock_file_name = "get_mint_token_accounts"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetMintTokenAccountsResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_mint_token_accounts(JUP, page_size = 2)

        # actual test
        assert isinstance(response, GetMintTokenAccountsResponse)

    def test_get_on_chain_token_data_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get On Chain Token Data" for synchronous logic.

            Mock Response File: get_on_chain_token_data.json
        """

        # load mock response
        mock_file_name = "get_on_chain_token_data"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetOnChainTokenDataResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        CWIF = "7atgF8KQo4wJrD5ATGX7t1V2zVvykPJbFfNeVf1icFv1"
        response = self.solana_fm.client.get_on_chain_token_data(CWIF)

        # actual test
        assert isinstance(response, GetOnChainTokenDataResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_on_chain_token_data_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get On Chain Token Data" for asynchronous logic.

            Mock Response File: get_on_chain_token_data.json
        """

        # load mock response
        mock_file_name = "get_on_chain_token_data"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetOnChainTokenDataResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        CWIF = "7atgF8KQo4wJrD5ATGX7t1V2zVvykPJbFfNeVf1icFv1"
        async with self.solana_fm.async_client as client:
            response = await client.get_on_chain_token_data(CWIF)

        # actual test
        assert isinstance(response, GetOnChainTokenDataResponse)

    def test_get_token_supply_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Token Supply" for synchronous logic.

            Mock Response File: get_token_supply.json
        """

        # load mock response
        mock_file_name = "get_token_supply"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTokenSupplyResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_token_supply(JUP)

        # actual test
        assert isinstance(response, GetTokenSupplyResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_token_supply_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Token Supply" for asynchronous logic.

            Mock Response File: get_token_supply.json
        """

        # load mock response
        mock_file_name = "get_token_supply"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTokenSupplyResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_token_supply(JUP)

        # actual test
        assert isinstance(response, GetTokenSupplyResponse)

    def test_get_transfer_transactions_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Transfer Transactions" for synchronous logic.

            Mock Response File: get_transfer_transactions.json
        """

        # load mock response
        mock_file_name = "get_transfer_transactions"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTransferTransactionsResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        transaction = "FUCoNwzCfPugjBwR4Q2uzDMZJdA3E5GAqDB1HGgfi3cTPyqBk11HniqQCGSCin3oBGQLFwcQ3fEHdsEJbgaZZ5p"
        response = self.solana_fm.client.get_transfer_transactions(transaction)

        # actual test
        assert isinstance(response, GetTransferTransactionsResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_transfer_transactions_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get Transfer Transactions" for asynchronous logic.

            Mock Response File: get_transfer_transactions.json
        """

        # load mock response
        mock_file_name = "get_transfer_transactions"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetTransferTransactionsResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        transaction = "FUCoNwzCfPugjBwR4Q2uzDMZJdA3E5GAqDB1HGgfi3cTPyqBk11HniqQCGSCin3oBGQLFwcQ3fEHdsEJbgaZZ5p"
        async with self.solana_fm.async_client as client:
            response = await client.get_transfer_transactions(transaction)

        # actual test
        assert isinstance(response, GetTransferTransactionsResponse)

    def test_post_multiple_transfer_transactions_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Multiple Transfer Transactions" for synchronous logic.

            Mock Response File: post_multiple_transfer_transactions.json
        """

        # load mock response
        mock_file_name = "post_multiple_transfer_transactions"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostMultipleTransferTransactionsResponse)
            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        transactions = [
            "4wGMriaRA55F1opEhJW2is34aDQdJHE13vJCbqP6WzrnzFwThnvZYvRZ2AKc2Meh5JToETz3eCCeuavepz6iTPZx",
            "FUCoNwzCfPugjBwR4Q2uzDMZJdA3E5GAqDB1HGgfi3cTPyqBk11HniqQCGSCin3oBGQLFwcQ3fEHdsEJbgaZZ5p"
        ]
        response = self.solana_fm.client.post_multiple_transfer_transactions(transactions)

        # actual test
        assert isinstance(response, PostMultipleTransferTransactionsResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_post_multiple_transfer_transactions_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Post Multiple Transfer Transactions" for asynchronous logic.

            Mock Response File: post_multiple_transfer_transactions.json
        """

        # load mock response
        mock_file_name = "post_multiple_transfer_transactions"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, PostMultipleTransferTransactionsResponse)
            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        transactions = [
            "4wGMriaRA55F1opEhJW2is34aDQdJHE13vJCbqP6WzrnzFwThnvZYvRZ2AKc2Meh5JToETz3eCCeuavepz6iTPZx",
            "FUCoNwzCfPugjBwR4Q2uzDMZJdA3E5GAqDB1HGgfi3cTPyqBk11HniqQCGSCin3oBGQLFwcQ3fEHdsEJbgaZZ5p"
        ]
        async with self.solana_fm.async_client as client:
            response = await client.post_multiple_transfer_transactions(transactions)

        # actual test
        assert isinstance(response, PostMultipleTransferTransactionsResponse)

    def test_get_all_transfer_actions_sync(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get All Transfer Actions" for synchronous logic.

            Mock Response File: get_all_transfer_actions.json
        """

        # load mock response
        mock_file_name = "get_all_transfer_actions"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAllTransferActionsResponse)

            # response content to be adjusted
            content = self.mocker.adjust_content_json(str(mock_response.json()["actions"]))
            mock_response._content = content

            mocker.patch("cyhole.core.client.APIClient.api", return_value = mock_response)

        # execute request
        response = self.solana_fm.client.get_all_transfer_actions()

        # actual test
        assert isinstance(response, GetAllTransferActionsResponse)

        # store request (only not mock)
        if config.mock_file_overwrite and not config.solana_fm.mock_response:
            self.mocker.store_mock_model(mock_file_name, response)

    @pytest.mark.asyncio
    async def test_get_all_transfer_actions_async(self, mocker: MockerFixture) -> None:
        """
            Unit Test used to check the response schema of endpoint 
            "Get All Transfer Actions" for asynchronous logic.

            Mock Response File: get_all_transfer_actions.json
        """

        # load mock response
        mock_file_name = "get_all_transfer_actions"
        if config.mock_response or config.solana_fm.mock_response:
            mock_response = self.mocker.load_mock_response(mock_file_name, GetAllTransferActionsResponse)

            # response content to be adjusted
            content = self.mocker.adjust_content_json(str(mock_response.json()["actions"]))
            mock_response._content = content

            mocker.patch("cyhole.core.client.AsyncAPIClient.api", return_value = mock_response)

        # execute request
        async with self.solana_fm.async_client as client:
            response = await client.get_all_transfer_actions()

        # actual test
        assert isinstance(response, GetAllTransferActionsResponse)


from datetime import datetime
from pydantic import BaseModel, Field, field_serializer, field_validator

class SolanaFMBaseResponse(BaseModel):
    """
        Model used to identify the base response of the SolanaFM API.
    """
    status: str
    message: str

class SolanaFMPagination(BaseModel):
    """
        Model used to identify the pagination of the SolanaFM API.
    """
    current_page: int = Field(alias = "currentPage")
    total_pages: int = Field(alias = "totalPages")

# classes used on GET "Account - Transactions" endpoint
# Param
class GetAccountTransactionsParam(BaseModel):
    """
        Model used to identify the parameters of the GET "Account - Transactions" endpoint.
    """

    actions: str | None = None
    """The action(s) to filter by."""

    utc_from_unix_time: int | None = Field(default = None, serialization_alias = "utcFrom")
    """The start date of the transactions to filter by in Unix time."""

    utc_to_unix_time: int | None = Field(default = None, serialization_alias = "utcTo")
    """The end date of the transactions to filter by in Unix time."""

    inflow: bool | None = None
    """Whether to include inflow transactions."""

    outflow: bool | None = None
    """Whether to include outflow transactions."""

    mints: str | list[str] | None = None
    """The mint ID(s) to filter by."""

    amount_from: int | None = Field(default = None, serialization_alias = "amountFrom", gt = 0)
    """The minimum amount of the transaction."""

    amount_to: int | None = Field(default = None, serialization_alias = "amountTo", gt = 0)
    """The maximum amount of the transaction."""

    programs: str | list[str] | None = None
    """The program ID(s) to filter by."""

    limit: int = Field(default = 1, le = 1000, gt = 0)
    """The number of transactions to return."""

    page: int = Field(default = 1, gt = 0)
    """The page number to return."""

    @field_serializer("mints", "programs")
    @classmethod
    def serialize_str_list_input(cls, str_input: str | list[str] | None) -> str | None:
        return ",".join(str_input) if str_input else None

# Response
class GetAccountTransactionsData(BaseModel):
    """
        Model used to identify the data of the GET "Account - Transactions" endpoint.
    """
    block_time: int = Field(alias = "blockTime")
    confirmation_status: str = Field(alias = "confirmationStatus")
    error: str | None = Field(default = None, alias = "err")
    memo: str | None = None
    signature: str
    slot: int

class GetAccountTransactionsResult(BaseModel):
    """
        Model used to identify the result of the GET "Account - Transactions" endpoint.
    """
    data: list[GetAccountTransactionsData]
    pagination: SolanaFMPagination

class GetAccountTransactionsResponse(SolanaFMBaseResponse):
    """
        Model used to identify the response of the GET "Account - Transactions" endpoint.
    """
    result: GetAccountTransactionsResult

# classes used on GET "Account - Transfers" endpoint
# Param
class GetAccountTransfersParam(BaseModel):
    """
        Model used to identify the parameters of the GET "Account - Transfers" endpoint.
    """
    utc_from_unix_time: int | None = Field(default = None, serialization_alias = "utcFrom")
    """The start date of the transactions to filter by in Unix time."""

    utc_to_unix_time: int | None = Field(default = None, serialization_alias = "utcTo")
    """The end date of the transactions to filter by in Unix time."""

    inflow: bool | None = None
    """Whether to include inflow transactions."""

    outflow: bool | None = None
    """Whether to include outflow transactions."""

    mint: str | list[str] | None = None
    """The mint ID(s) to filter by."""

    limit: int = Field(default = 1, le = 100, gt = 0)
    """The number of transactions to return."""

    page: int = Field(default = 1, gt = 0)
    """The page number to return."""

    @field_serializer("mint")
    @classmethod
    def serialize_str_list_input(cls, str_input: str | list[str] | None) -> str | None:
        return ",".join(str_input) if str_input else None

# Response
class GetAccountTransfersData(BaseModel):
    """
        Model used to identify the data of the GET "Account - Transfers" endpoint.
    """
    instruction_index: int = Field(alias = "instructionIndex")
    inner_instruction_index: int = Field(alias = "innerInstructionIndex")
    action: str
    status: str
    source: str | None = None
    source_association: str | None = Field(default = None, alias = "sourceAssociation")
    destination: str | None = None
    destination_association: str | None = Field(default = None, alias = "destinationAssociation")
    token: str
    amount: int
    timestamp: int

class GetAccountTransfersResults(BaseModel):
    """
        Model used to identify the results of the GET "Account - Transfers" endpoint.
    """
    transaction_hash: str = Field(alias = "transactionHash")
    data: list[GetAccountTransfersData]

class GetAccountTransfersResponse(SolanaFMBaseResponse):
    """
        Model used to identify the response of the GET "Account - Transfers" endpoint.
    """
    results: list[GetAccountTransfersResults]
    pagination: SolanaFMPagination

# classes used on GET "Account - Transfers CSV Export" endpoint
# Param
class GetAccountTransfersCsvExportParam(GetAccountTransactionsParam):
    """
        Model used to identify the parameters of the GET "Account - Transfers CSV Export" endpoint.
    """
    pass

# Response
class GetAccountTransfersCsvExportResponse(BaseModel):
    """
        Model used to identify the response of the GET "Account - Transfers CSV Export" endpoint.
    """
    csv: str

# classes used on GET "Account Transactions Fees" endpoint
# Response
class GetAccountTransactionsFeesData(BaseModel):
    """
        Model used to identify the data of the GET "Account Transactions Fees" endpoint.
    """
    tx_fees: int
    time: datetime

    @field_validator("time")
    @classmethod
    def parse_time(cls, time: str | datetime) -> datetime:
        if isinstance(time, str):
            return datetime.strptime(time, "%Y-%m-%d")
        return time

class GetAccountTransactionsFeesResponse(BaseModel):
    """
        Model used to identify the response of the GET "Account Transactions Fees" endpoint.
    """
    data: list[GetAccountTransactionsFeesData]

# classes used on GET "Blocks" endpoint
class SolanaFMBlockData(BaseModel):
    """
        Model used to identify the data of the SolanaFM block.
    """
    epoch: int
    previous_hash: str = Field(alias = "previousHash")
    hash: str
    parent_number: int = Field(alias = "parentNumber")
    number: int
    data_size: int = Field(alias = "dataSize")
    number_of_transactions: int = Field(alias = "numberOfTransactions")
    successful_transactions: int = Field(alias = "successfulTransactions")
    vote_transactions: int = Field(alias = "voteTransactions")
    total_tx_fees: int = Field(alias = "totalTxFees")
    number_of_rewards: int = Field(alias = "numberOfRewards")
    total_reward_amount: int = Field(alias = "totalRewardAmount")
    total_compute_units_consumed: int = Field(alias = "totalComputeUnitsConsumed")
    total_compute_units_limit: int = Field(alias = "totalComputeUnitsLimit")
    block_time: int = Field(alias = "blockTime")

# Response
class GetBlocksDataData(SolanaFMBlockData):
    """
        Model used to identify the info of the GET "Blocks" endpoint.
    """
    producer: str

class GetBlocksData(BaseModel):
    """
        Model used to identify the data of the GET "Blocks" endpoint.
    """
    block_number: int = Field(alias = "blockNumber")
    data: GetBlocksDataData

class GetBlocksPagination(BaseModel):
    """
        Model used to identify the pagination of the GET "Blocks" endpoint.
    """
    next: int | None = None
    previous: int | None = None

class GetBlocksResult(BaseModel):
    """
        Model used to identify the result of the GET "Blocks" endpoint.
    """
    data: list[GetBlocksData]
    pagination: GetBlocksPagination

class GetBlocksResponse(SolanaFMBaseResponse):
    """
        Model used to identify the data of the GET "Blocks" endpoint.
    """
    result: GetBlocksResult

# classes used on GET "Block" endpoint
# Response
class GetBlockResult(GetBlocksData):
    """
        Model used to identify the result of the GET "Block" endpoint.
    """
    pass

class GetBlockResponse(SolanaFMBaseResponse):
    """
        Model used to identify the response of the GET "Block" endpoint.
    """
    result: GetBlockResult

# classes used on POST "Multiple Blocks" endpoint
# Response
class PostMultipleBlocksProducerData(BaseModel):
    """
        Model used to identify the data of the producer from POST "Multiple Blocks" endpoint.
    """
    friendly_name: str = Field(alias = "friendlyName")
    abbreviation: str
    category: str
    vote_key: str = Field(alias = "voteKey")
    network: str
    tags: list[str]
    logo_uri: str | None = Field(default = None, alias = "logoURI")
    flag: str | None = None

class PostMultipleBlocksProducer(BaseModel):
    """
        Model used to identify the producer of the POST "Multiple Blocks" endpoint.
    """
    account_hash: str = Field(alias = "accountHash")
    data: PostMultipleBlocksProducerData

class PostMultipleBlocksData(SolanaFMBlockData):
    """
        Model used to identify the data of the POST "Multiple Blocks" endpoint.
    """
    producer: str | PostMultipleBlocksProducer

class PostMultipleBlocksResult(BaseModel):
    """
        Model used to identify the result of the POST "Multiple Blocks" endpoint.
    """
    block_number: int = Field(alias = "blockNumber")
    data: PostMultipleBlocksData

class PostMultipleBlocksResponse(SolanaFMBaseResponse):
    """
        Model used to identify the response of the POST "Multiple Blocks" endpoint.
    """
    result: list[PostMultipleBlocksResult]

# classes used on GET "Solana Daily Transaction Fees" endpoint
# Response
class GetSolanaDailyTransactionFeesResult(BaseModel):
    """
        Model used to identify the result of the GET "Solana Daily Transaction Fees" endpoint.
    """
    total_tx_fees: int = Field(alias = "totalTxFees")
    date: str

    @field_validator("date")
    @classmethod
    def parse_date(cls, dt: str | datetime) -> datetime:
        if isinstance(dt, str):
            return datetime.strptime(dt, "%d-%m-%Y")
        return dt

    @field_serializer("date")
    @classmethod
    def serialize_date(cls, dt: datetime) -> str:
        return dt.strftime("%d-%m-%Y")

class GetSolanaDailyTransactionFeesResponse(SolanaFMBaseResponse):
    """
        Model used to identify the response of the GET "Solana Daily Transaction Fees" endpoint.
    """
    result: GetSolanaDailyTransactionFeesResult

# classes used on GET "Tagged Tokens List" endpoint
# Response
class GetTaggedTokensListPagination(BaseModel):
    """
        Model used to identify the pagination of the GET "Blocks" endpoint.
    """
    next: str | None = None
    previous: str | None = None

class GetTaggedTokensListDataData(BaseModel):
    """
        Model used to identify the data of a token inside the GET "Tagged Tokens List" endpoint.
    """
    mint: str
    token_name: str = Field(alias = "tokenName")
    symbol: str
    decimals: int
    description: str
    logo: str
    tags: list[str]
    verified: str
    network: list[str]
    metadata_token: str = Field(alias = "metadataToken")

class GetTaggedTokensListData(BaseModel):
    """
        Model used to identify the data of the GET "Tagged Tokens List" endpoint.
    """
    token: str = Field(alias = "tokenHash")
    data: GetTaggedTokensListDataData

class GetTaggedTokensListResult(BaseModel):
    """
        Model used to identify the result of the GET "Tagged Tokens List" endpoint.
    """
    data: list[GetTaggedTokensListData]
    pagination: GetTaggedTokensListPagination

class GetTaggedTokensListResponse(SolanaFMBaseResponse):
    """
        Model used to identify the response of the GET "Tagged Tokens List" endpoint.
    """
    result: GetTaggedTokensListResult

# classes used on GET "Token Info (v0)" endpoint
# Response
class GetTokenInfoV0Result(GetTaggedTokensListData):
    """
        Model used to identify the data of the GET "Token Info (v0)" endpoint.
    """
    pass

class GetTokenInfoV0Response(SolanaFMBaseResponse):
    """
        Model used to identify the response of the GET "Token Info (v0)" endpoint.
    """
    result: GetTokenInfoV0Result

# classes used on POST "Token Multiple Info (v0)" endpoint
# Response
class PostTokenMultipleInfoV0Result(GetTaggedTokensListData):
    """
        Model used to identify the data of the POST "Token Multiple Info (v0)" endpoint
    """
    pass

class PostTokenMultipleInfoV0Response(SolanaFMBaseResponse):
    """
        Model used to identify the response of the POST "Token Multiple Info (v0)" endpoint.
    """
    result: list[PostTokenMultipleInfoV0Result]

# classes used on GET "Token Info (v1)" endpoint
# Response
class GetTokenInfoV1ChainInfo(BaseModel):
    """
        Model used to identify the chain metadata of the token in the GET "Token Info (v1)" endpoint.
    """
    name: str
    symbol: str
    metadata: str
    update_authority: str = Field(alias = "updateAuthority")
    is_master_edition: bool | None = Field(default = None, alias = "isMasterEdition")
    edition: str | None = None
    uri: str
    seller_fee_basis_points: int = Field(alias = "sellerFeeBasisPoints")
    primary_sale_happened: bool = Field(alias = "primarySaleHappened")
    is_mutable: bool = Field(alias = "isMutable")
    creators: list[str]
    rule_set: str | None = Field(default = None, alias = "ruleSet")
    collection: str | None = None
    collection_details: str | None = Field(default = None, alias = "collectionDetails")
    uses: str | None = None

class GetTokenInfoV1OffChainInfo(GetTokenInfoV1ChainInfo):
    """
        Model used to identify the off-chain metadata of the token in the GET "Token Info (v1)" endpoint.
    """
    pass

class GetTokenInfoV1OnChainInfo(GetTokenInfoV1ChainInfo):
    """
        Model used to identify the on-chain metadata of the token in the GET "Token Info (v1)" endpoint.
    """
    pass

class GetTokenInfoV1TokenMetadata(BaseModel):
    """
        Model used to identify the metadata of the token in the GET "Token Info (v1)" endpoint.
    """
    on_chain_info: GetTokenInfoV1OnChainInfo | None = Field(default = None, alias = "onChainInfo")
    off_chain_info: GetTokenInfoV1OffChainInfo | None = Field(default = None, alias = "offChainInfo")

class GetTokenInfoV1TokenList(BaseModel):
    """
        Model used to identify the token list in the GET "Token Info (v1)" endpoint.
    """
    name: str
    symbol: str
    image: str
    extensions: dict[str, str]
    chain_id: int = Field(alias = "chainId")

class GetTokenInfoV1Response(BaseModel):
    """
        Model used to identify the response of the GET "Token Info (v0)" endpoint.
    """
    mint: str
    decimals: int
    freeze_authority: str | None = Field(default = None, alias = "freezeAuthority")
    mint_authority: str | None = Field(default = None, alias = "mintAuthority")
    token_type: str = Field(alias = "tokenType")
    token_list: GetTokenInfoV1TokenList = Field(alias = "tokenList")
    token_metadata: GetTokenInfoV1TokenMetadata = Field(alias = "tokenMetadata")

# classes used on POST "Token Multiple Info (v1)" endpoint
# Response
class PostTokenMultipleInfoV1Response(BaseModel):
    """
        Model used to identify the response of the POST "Token Multiple Info (v1)" endpoint.
    """
    tokens: dict[str, GetTokenInfoV1Response]

# classes used on POST "User's Token Accounts" endpoint
# Response
class PostUserTokenAccountsInfoExtensionStateTransferFee(BaseModel):
    """
        Model used to identify the transfer fee of the extension state of the token in the POST "User's Token Accounts" endpoint.
    """
    epoch: int
    maximum_fee: int = Field(alias = "maximumFee")
    transfer_fee_basis_points: int = Field(alias = "transferFeeBasisPoints")

class PostUserTokenAccountsInfoExtensionState(BaseModel):
    """
        Model used to identify the state of the extension of the token in the POST "User's Token Accounts" endpoint.
    """
    newer_transfer_fee: PostUserTokenAccountsInfoExtensionStateTransferFee | None = Field(default = None, alias = "newerTransferFee")
    older_transfer_fee: PostUserTokenAccountsInfoExtensionStateTransferFee | None = Field(default = None, alias = "olderTransferFee")
    withheld_amount: int | None = Field(default = None, alias = "withheldAmount")
    withdraw_withheld_authority: str | None = Field(default = None, alias = "withdrawWithheldAuthority")
    transfer_fee_config_authority: str | None = Field(default = None, alias = "transferFeeConfigAuthority")

class PostUserTokenAccountsInfoExtension(BaseModel):
    """
        Model used to identify the extension of the token in the POST "User's Token Accounts" endpoint.
    """
    extension: str
    state: PostUserTokenAccountsInfoExtensionState | None = None

class PostUserTokenAccountsInfoTokenAmount(BaseModel):
    """
        Model used to identify the token amount of the token in the POST "User's Token Accounts" endpoint.
    """
    amount: int
    decimals: int
    ui_amount: float = Field(alias = "uiAmount")
    ui_amount_string: str = Field(alias = "uiAmountString")

class PostUserTokenAccountsInfo(BaseModel):
    """
        Model used to identify the info of the token of the POST "User's Token Accounts" endpoint.
    """
    extensions: list[PostUserTokenAccountsInfoExtension] | None = None
    is_native: bool = Field(alias = "isNative")
    mint: str
    owner: str
    state: str
    token_amount: PostUserTokenAccountsInfoTokenAmount = Field(alias = "tokenAmount")

class PostUserTokenAccounts(BaseModel):
    """
        Model used to identify the token's account of the POST "User's Token Accounts" endpoint.
    """
    id: str = Field(alias = "_id")
    info: PostUserTokenAccountsInfo
    slot: int

class PostUserTokenAccountsResponse(BaseModel):
    """
        Model used to identify the response of the POST "User's Token Accounts" endpoint.
    """
    sol_balance: int | None = Field(default = None, alias = "solBalance")
    token_accounts: list[PostUserTokenAccounts] = Field(alias = "tokenAccounts")

# classes used on GET "Mint Token Accounts" endpoint
# Response
class GetMintTokenAccountsTokenAccounts(PostUserTokenAccounts):
    """
        Model used to identify the token accounts of the GET "Mint Token Accounts" endpoint.
    """
    pass

class GetMintTokenAccountsResponse(BaseModel):
    """
        Model used to identify the response of the GET "Mint Token Accounts" endpoint.
    """
    token_accounts: list[GetMintTokenAccountsTokenAccounts] = Field(alias = "tokenAccounts")
    total_item_count: int = Field(alias = "totalItemCount")

# classes used on GET "On-Chain Token Data" endpoint
# Response
class GetOnChainTokenDataInfoExtension(PostUserTokenAccountsInfoExtension):
    """
        Model used to identify the extension of the token in the GET "On-Chain Token Data" endpoint.
    """
    pass

class GetOnChainTokenDataInfo(BaseModel):
    """
        Model used to identify the info of the token in the GET "On-Chain Token Data" endpoint.
    """
    decimals: int
    extensions: list[GetOnChainTokenDataInfoExtension] | None = None
    freeze_authority: str | None = Field(default = None, alias = "freezeAuthority")
    is_initialized: bool = Field(alias = "isInitialized")
    mint_authority: str | None = Field(default = None, alias = "mintAuthority")
    supply: str

class GetOnChainTokenDataResponse(BaseModel):
    """
        Model used to identify the response of the GET "On-Chain Token Data" endpoint.
    """
    id: str
    info: GetOnChainTokenDataInfo
    slot: int
    type: str | None = None

# classes used on GET "Token Supply" endpoint
# Response
class GetTokenSupplyResponse(BaseModel):
    """
        Model used to identify the response of the GET "Token Supply" endpoint.
    """
    circulating_supply: float = Field(alias = "circulatingSupply")
    token_withheld_amount: int | None = Field(default = None, alias = "tokenWithheldAmount")
    user_total_withheld_amount: int = Field(alias = "userTotalWithheldAmount")
    total_withheld_amount: int = Field(alias = "totalWithheldAmount")
    real_circulating_supply: float = Field(alias = "realCirculatingSupply")
    decimals: int

# classes used on GET "Transfer Transactions" endpoint
# Response
class GetTransferTransactionsResult(GetAccountTransfersResults):
    """
        Model used to identify the result of the GET "Transfer Transactions" endpoint.
    """
    pass

class GetTransferTransactionsResponse(SolanaFMBaseResponse):
    """
        Model used to identify the response of the GET "Transfer Transactions" endpoint.
    """
    result: GetTransferTransactionsResult

# classes used on POST "Multiple Transfer Transactions" endpoint
# Response
class PostMultipleTransferTransactionsResult(GetAccountTransfersResults):
    """
        Model used to identify the result of the POST "Multiple Transfer Transactions" endpoint.
    """
    pass

class PostMultipleTransferTransactionsResponse(SolanaFMBaseResponse):
    """
        Model used to identify the response of the POST "Multiple Transfer Transactions" endpoint.
    """
    result: list[PostMultipleTransferTransactionsResult]

# classes used on GET "All Transfer Actions" endpoint
# Response
class GetAllTransferActionsResponse(BaseModel):
    """
        Model used to identify the response of the GET "All Transfer Actions" endpoint.
    """
    actions: list[str]


from typing import Any, TypeVar, Type, Coroutine
from pydantic import BaseModel

from ..core.client import APIClient, AsyncAPIClient

ResponseModel = TypeVar('ResponseModel', bound = BaseModel)

class Interaction:
    """
        This class is the building block for a new integration of an external API in `cyhole` library, 
        and every new external API **must** be identified by a class that inherits from this class.

        Inside an `Interaction` there are two attributes identifying the clients responsable to 
        perform the API calls:

        - `client`: object used for **synchronous** logic.
        - `async_client`: object used for **asynchronous** logic.

        During the creation of the object is possible to specify some global configurations.

        Parameters:
            headers: headers used globally in all API requests.
    """
    def __init__(self, headers: Any | None = None) -> None:
        self.headers = headers

        # clients
        self.client = APIClient(self, headers = self.headers)
        self.async_client = AsyncAPIClient(self, headers = self.headers)

        return

    def api_return_model(self, sync: bool, type: str, url: str, response_model: Type[ResponseModel], *args: tuple, **kwargs: dict[str, Any]) -> ResponseModel | Coroutine[None, None, ResponseModel]:
        """
            This function is used to execute a request to the API by forwarding the call to the 
            corresponding client (synchronous or asynchronous) according to the `sync` parameter.

            This function is used to avoid code duplication in the methods of the `Interaction` classes, 
            and it assumes that the response of the API request is a JSON object that can be parsed into 
            a `pydantic.BaseModel` model.

            Parameters:
                sync: boolean to define if the request is synchronous or asynchronous.
                type: request's type ([`RequestType`][cyhole.core.param.RequestType]).
                url: the URL of the API endpoint.
                response_model: the `pydantic.BaseModel` model used to parse the response.
                args: the positional arguments to be passed to the API request.
                kwargs: the named arguments to be passed to the API request.

            Returns:
                The response of the API request. If the request is synchronous, the response is a 
                `pydantic.BaseModel` model. If the request is asynchronous, the response is a coroutine 
                that will return a `pydantic.BaseModel` model.
        """
        if sync:
            content_raw = self.client.api(type, url, *args, **kwargs)
            return response_model(**content_raw.json())
        else:
            async def async_request():
                content_raw = await self.async_client.api(type, url, *args, **kwargs)
                return response_model(**content_raw.json())
            return async_request()


import json
import configparser
from pathlib import Path
from requests import Response
from typing import TypeVar, Type
from pydantic import BaseModel

ResponseModel = TypeVar('ResponseModel', bound = BaseModel)

class SolanaFMConfiguration(BaseModel):
    """
        Model in charge to manage the SolanaFM APIs.
    """
    mock_response: bool = True
    """Flag to enable/disable the mock responses."""
    mock_folder: str = "solana_fm"
    """Folder where the mock responses are stored."""
    api_key: str | None = None
    """API key to access the SolanaFM APIs."""

class JupiterConfiguration(BaseModel):
    """
        Model in charge to manage the Jupiter APIs.
    """
    mock_response: bool = True
    """Flag to enable/disable the mock responses."""
    mock_folder: str = "jupiter"
    """Folder where the mock responses are stored."""

class BirdeyeConfiguration(BaseModel):
    """
        Model in charge to manage the birdeye APIs.
    """
    mock_response_public: bool = True
    """Flag to enable/disable the mock responses for the public APIs."""
    mock_response_private: bool = True
    """Flag to enable/disable the mock responses for the private APIs."""
    mock_folder: str = "birdeye"
    """Folder where the mock responses are stored."""
    api_key: str = ""
    """API key to access the birdeye APIs."""

class TestConfiguration(BaseModel):
    """
        Model in charge to manage the tests' configuration.
    """
    mock_response: bool = True
    """Flag to enable/disable the mock responses."""
    mock_folder: str = "tests/resources/mock"
    """Folder where the mock responses are stored."""
    mock_file_overwrite: bool = False
    """Flag to enable/disable the overwrite of the mock files."""

    birdeye: BirdeyeConfiguration = BirdeyeConfiguration()
    """Birdeye configuration."""
    jupiter: JupiterConfiguration = JupiterConfiguration()
    """Jupiter configuration."""
    solana_fm: SolanaFMConfiguration = SolanaFMConfiguration()
    """SolanaFM configuration."""

def load_config(path: str = "tests", file: str = "test.ini") -> TestConfiguration:
    """
        This function is used to load the test configuration file.

        It is **strongly suggested** to leave the default `path` and `file` 
        name for the test configuration file. However, if a dedicated file 
        is used, then is possible to load another file by providing the 
        necessary information in the input variables.

        Parameters:
            path: path of the configuration file.
            file: name of the configuration file (with extension).

        Returns:
            pydantic model of the test's configuration.
    """
    config_path_file = f"{path}/{file}"
    config = configparser.ConfigParser()

    # load config
    config.read(config_path_file)
    test_config = TestConfiguration()
    
    # global
    test_config.mock_response = config.getboolean("global", "mock_response", fallback = test_config.mock_response)
    test_config.mock_folder = config.get("global", "mock_folder", fallback = test_config.mock_folder)
    test_config.mock_file_overwrite = config.getboolean("global", "mock_file_overwrite", fallback = test_config.mock_file_overwrite)

    # birdeye
    test_config.birdeye.mock_response_public = config.getboolean("birdeye", "mock_response_public", fallback = test_config.birdeye.mock_response_public)
    test_config.birdeye.mock_response_private = config.getboolean("birdeye", "mock_response_private", fallback = test_config.birdeye.mock_response_private)
    test_config.birdeye.mock_folder = config.get("birdeye", "mock_folder", fallback = test_config.birdeye.mock_folder)
    test_config.birdeye.api_key = config.get("birdeye", "api_key", fallback = test_config.birdeye.api_key)

    # jupiter
    test_config.jupiter.mock_response = config.getboolean("jupiter", "mock_response", fallback = test_config.jupiter.mock_response)
    test_config.jupiter.mock_folder = config.get("jupiter", "mock_folder", fallback = test_config.jupiter.mock_folder)

    # solana_fm
    test_config.solana_fm.mock_response = config.getboolean("solana_fm", "mock_response", fallback = test_config.solana_fm.mock_response)
    test_config.solana_fm.mock_folder = config.get("solana_fm", "mock_folder", fallback = test_config.solana_fm.mock_folder)
    test_config.solana_fm.api_key = config.get("solana_fm", "api_key", fallback = test_config.solana_fm.api_key)

    return test_config

class MockerManager:
    """
        Class used to manage the mock responses.
    """

    def __init__(self, mock_path: Path) -> None:
        # set/create resources folder
        self.mock_path = mock_path
        mock_path.mkdir(parents = True, exist_ok = True)
    
    def load_mock_model(self, file_name: str, response_model: Type[ResponseModel]) -> ResponseModel:
        """
            Use this function to load mock response model from a file.

            Parameters:
                file_name: file name containing the response to load.
                    By assumption, the file must be a JSON file and the file name should not contain the extension.
                response_model: objects that inherit from `pydantic.BaseModel` and describe the response object element.
        """
        mock_file = f"{file_name}.json"
        mock_path_file = self.mock_path / mock_file

        if not mock_path_file.exists():
            raise Exception(f"mock file '{mock_file}' does not exist in '{self.mock_path}'.")

        with open(mock_path_file, "r") as file:
            data = json.loads(file.read())
            mock_response = response_model(**data)

        return mock_response

    def store_mock_model(self, file_name: str, response: BaseModel) -> None:
        """
            Use this function to store the mock response model into a file.

            Parameters:
                file_name: file name containing the response to load.
                    By assumption, the file must be a JSON file and the file name should not contain the extension.
                response: objects that inherit from `pydantic.BaseModel` and describe the response object element.
        """
        mock_file = f"{file_name}.json"
        mock_path_file = self.mock_path / mock_file

        with open(mock_path_file, "w") as file:
            file.write(response.model_dump_json(indent = 4, by_alias = True))

        return

    def load_mock_response(self, file_name: str, response_model: Type[ResponseModel]) -> Response:
        """
            Use this function to load mock response model from a file and return a dummy `Response` object.

            Parameters:
                file_name: file name containing the response to load.
                    By assumption, the file must be a JSON file and the file name should not contain the extension.
                response_model: objects that inherit from `pydantic.BaseModel` and describe the response object element.
        """
        mock_model = self.load_mock_model(file_name, response_model)

        # create dummy response
        mock_response = Response()
        mock_response._content = mock_model.model_dump_json(by_alias = True, exclude_none = True).encode()
        mock_response.status_code = 200
        mock_response.encoding = "utf-8"

        return mock_response

    def adjust_content_json(self, content: str) -> bytes:
        """
            Use this function to adjust an input string to ensure that 
            can be used as content for a `Response` object and paresed 
            as a JSON object.

            Parameters:
                content: string to adjust.
        """
        # adjustemts
        content = content.replace("'", '"')
        content = content.replace("False", "false")
        content = content.replace("True", "true")
        content = content.replace("None", "null")

        return content.encode()


from __future__ import annotations
from datetime import datetime
from typing import TYPE_CHECKING, Any

from ..core.client import APIClient, AsyncAPIClient
from ..solana_fm.param import SolanaFMBlocksPaginationType
from ..solana_fm.schema import (
    GetAccountTransactionsParam,
    GetAccountTransactionsResponse,
    GetAccountTransfersParam,
    GetAccountTransfersResponse,
    GetAccountTransfersCsvExportParam,
    GetAccountTransfersCsvExportResponse,
    GetAccountTransactionsFeesResponse,
    GetBlocksResponse,
    GetBlockResponse,
    PostMultipleBlocksResponse,
    GetSolanaDailyTransactionFeesResponse,
    GetTaggedTokensListResponse,
    GetTokenInfoV0Response,
    PostTokenMultipleInfoV0Response,
    GetTokenInfoV1Response,
    PostTokenMultipleInfoV1Response,
    PostUserTokenAccountsResponse,
    GetMintTokenAccountsResponse,
    GetOnChainTokenDataResponse,
    GetTokenSupplyResponse,
    GetTransferTransactionsResponse,
    PostMultipleTransferTransactionsResponse,
    GetAllTransferActionsResponse
)

if TYPE_CHECKING:
    from ..solana_fm.interaction import SolanaFM

class SolanaFMClient(APIClient):
    """
        Client used for synchronous API calls for `SolanaFM` interaction.
    """

    def __init__(self, interaction: SolanaFM, headers: Any | None = None) -> None:
        super().__init__(interaction, headers)
        self._interaction: SolanaFM = self._interaction

    def get_account_transactions(self, account: str, params: GetAccountTransactionsParam = GetAccountTransactionsParam()) -> GetAccountTransactionsResponse:
        """
            Call the SolanaFM's API endpoint GET **[Account Transactions](https://docs.solana.fm/reference/get_account_transactions)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_account_transactions`][cyhole.solana_fm.interaction.SolanaFM._get_account_transactions].
        """
        return self._interaction._get_account_transactions(True, account, params)

    def get_account_transactions_fees(self, account: str, dt_from: datetime | None = None, dt_to: datetime | None = None) -> GetAccountTransactionsFeesResponse:
        """
            Call the SolanaFM's API endpoint GET **[Account Transactions Fees](https://docs.solana.fm/reference/get_account_tx_fees)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_account_transactions_fees`][cyhole.solana_fm.interaction.SolanaFM._get_account_transactions_fees].
        """
        return self._interaction._get_account_transactions_fees(True, account, dt_from, dt_to)

    def get_account_transfers(self, account: str, params: GetAccountTransfersParam = GetAccountTransfersParam()) -> GetAccountTransfersResponse:
        """
            Call the SolanaFM's API endpoint GET **[Account Transfers](https://docs.solana.fm/reference/get_account_transfers_v1)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_account_transfers`][cyhole.solana_fm.interaction.SolanaFM._get_account_transfers].
        """
        return self._interaction._get_account_transfers(True, account, params)

    def get_account_transfers_csv_export(self, account: str, params: GetAccountTransfersCsvExportParam = GetAccountTransfersCsvExportParam()) -> GetAccountTransfersCsvExportResponse:
        """
            Call the SolanaFM's API endpoint GET **[Account Transfers CSV Export](https://docs.solana.fm/reference/download_csv_v1)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_account_transfers_csv_export`][cyhole.solana_fm.interaction.SolanaFM._get_account_transfers_csv_export].
        """
        return self._interaction._get_account_transfers_csv_export(True, account, params)

    def get_blocks(
        self,
        from_block: int | None = None,
        page_size: int = 50,
        page_type: str = SolanaFMBlocksPaginationType.BLOCK_NUMBER.value,
        ascending: bool | None = None
    ) -> GetBlocksResponse:
        """
            Call the SolanaFM's API endpoint GET **[Blocks](https://docs.solana.fm/reference/get_blocks_by_pagination)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_blocks`][cyhole.solana_fm.interaction.SolanaFM._get_blocks].
        """
        return self._interaction._get_blocks(True, from_block, page_size, page_type, ascending)

    def get_block(self, block_number: int) -> GetBlockResponse:
        """
            Call the SolanaFM's API endpoint GET **[Block](https://docs.solana.fm/reference/get_block)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_block`][cyhole.solana_fm.interaction.SolanaFM._get_block].
        """
        return self._interaction._get_block(True, block_number)

    def post_multiple_blocks(self, block_numbers: list[int], producer_details: bool = True) -> PostMultipleBlocksResponse:
        """
            Call the SolanaFM's API endpoint POST **[Multiple Blocks](https://docs.solana.fm/reference/get_multiple_blocks)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_multiple_blocks`][cyhole.solana_fm.interaction.SolanaFM._post_multiple_blocks].
        """
        return self._interaction._post_multiple_blocks(True, block_numbers, producer_details)

    def get_solana_daily_transaction_fees(self, dt: datetime = datetime.now()) -> GetSolanaDailyTransactionFeesResponse:
        """
            Call the SolanaFM's API endpoint GET **[Solana Daily Transaction Fees](https://docs.solana.fm/reference/get_daily_tx_fees)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_solana_daily_transaction_fees`][cyhole.solana_fm.interaction.SolanaFM._get_solana_daily_transaction_fees].
        """
        return self._interaction._get_solana_daily_transaction_fees(True, dt)

    def get_tagged_tokens_list(self) -> GetTaggedTokensListResponse:
        """
            Call the SolanaFM's API endpoint GET **[Tagged Tokens List](https://docs.solana.fm/reference/get_tokens_by_pagination)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_tagged_tokens_list`][cyhole.solana_fm.interaction.SolanaFM._get_tagged_tokens_list].
        """
        return self._interaction._get_tagged_tokens_list(True)

    def get_token_info_v0(self, address: str) -> GetTokenInfoV0Response:
        """
            Call the SolanaFM's API endpoint GET **[Token Info V0](https://docs.solana.fm/reference/get_token_by_account_hash)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_token_info_v0`][cyhole.solana_fm.interaction.SolanaFM._get_token_info_v0].
        """
        return self._interaction._get_token_info_v0(True, address)

    def post_token_multiple_info_v0(self, addresses: list[str]) -> PostTokenMultipleInfoV0Response:
        """
            Call the SolanaFM's API endpoint POST **[Token Multiple Info V0](https://docs.solana.fm/reference/get_tokens_by_account_hashes)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_token_multiple_info_v0`][cyhole.solana_fm.interaction.SolanaFM._post_token_multiple_info_v0].
        """
        return self._interaction._post_token_multiple_info_v0(True, addresses)

    def get_token_info_v1(self, address: str) -> GetTokenInfoV1Response:
        """
            Call the SolanaFM's API endpoint GET **[Token Info V1](https://docs.solana.fm/reference/get_one_token)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_token_info_v1`][cyhole.solana_fm.interaction.SolanaFM._get_token_info_v1].
        """
        return self._interaction._get_token_info_v1(True, address)

    def post_token_multiple_info_v1(self, addresses: list[str]) -> PostTokenMultipleInfoV1Response:
        """
            Call the SolanaFM's API endpoint POST **[Token Multiple Info V1](https://docs.solana.fm/reference/retrieve_multiple_tokens)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_token_multiple_info_v1`][cyhole.solana_fm.interaction.SolanaFM._post_token_multiple_info_v1].
        """
        return self._interaction._post_token_multiple_info_v1(True, addresses)

    def post_user_token_accounts(self, address: str, include_sol_balance: bool = False, tokens: list[str] | None = None) -> PostUserTokenAccountsResponse:
        """
            Call the SolanaFM's API endpoint POST **[User Token Accounts](https://docs.solana.fm/reference/get_user_token_accounts)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_user_token_accounts`][cyhole.solana_fm.interaction.SolanaFM._post_user_token_accounts].
        """
        return self._interaction._post_user_token_accounts(True, address, include_sol_balance, tokens)

    def get_mint_token_accounts(self, address: str, page: int | None = None, page_size: int | None = None) -> GetMintTokenAccountsResponse:
        """
            Call the SolanaFM's API endpoint GET **[Mint Token Accounts](https://docs.solana.fm/reference/get_token_accounts_for_token_mint)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_mint_token_accounts`][cyhole.solana_fm.interaction.SolanaFM._get_mint_token_accounts].
        """
        return self._interaction._get_mint_token_accounts(True, address, page, page_size)

    def get_on_chain_token_data(self, address: str) -> GetOnChainTokenDataResponse:
        """
            Call the SolanaFM's API endpoint GET **[On Chain Token Data](https://docs.solana.fm/reference/get_tfi_token_data)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_on_chain_token_data`][cyhole.solana_fm.interaction.SolanaFM._get_on_chain_token_data].
        """
        return self._interaction._get_on_chain_token_data(True, address)

    def get_token_supply(self, address: str) -> GetTokenSupplyResponse:
        """
            Call the SolanaFM's API endpoint GET **[Token Supply](https://docs.solana.fm/reference/get_token_circulating_supply)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_token_supply`][cyhole.solana_fm.interaction.SolanaFM._get_token_supply].
        """
        return self._interaction._get_token_supply(True, address)

    def get_transfer_transactions(self, transaction: str) -> GetTransferTransactionsResponse:
        """
            Call the SolanaFM's API endpoint GET **[Transfer Transactions](https://docs.solana.fm/reference/get_transfers)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_transfer_transactions`][cyhole.solana_fm.interaction.SolanaFM._get_transfer_transactions].
        """
        return self._interaction._get_transfer_transactions(True, transaction)

    def post_multiple_transfer_transactions(self, transactions: list[str]) -> PostMultipleTransferTransactionsResponse:
        """
            Call the SolanaFM's API endpoint POST **[Multiple Transfer Transactions](https://docs.solana.fm/reference/post_transfers)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_multiple_transfer_transactions`][cyhole.solana_fm.interaction.SolanaFM._post_multiple_transfer_transactions].
        """
        return self._interaction._post_multiple_transfer_transactions(True, transactions)

    def get_all_transfer_actions(self) -> GetAllTransferActionsResponse:
        """
            Call the SolanaFM's API endpoint GET **[All Transfer Actions](https://docs.solana.fm/reference/get_actions)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_all_transfer_actions`][cyhole.solana_fm.interaction.SolanaFM._get_all_transfer_actions].
        """
        return self._interaction._get_all_transfer_actions(True)

class SolanaFMAsyncClient(AsyncAPIClient):
    """
        Client used for asynchronous API calls for `SolanaFM` interaction.
    """

    def __init__(self, interaction: SolanaFM, headers: Any | None = None) -> None:
        super().__init__(interaction, headers)
        self._interaction: SolanaFM = self._interaction

    async def get_account_transactions(self, account: str, params: GetAccountTransactionsParam = GetAccountTransactionsParam()) -> GetAccountTransactionsResponse:
        """
            Call the SolanaFM's API endpoint GET **[Account Transactions](https://docs.solana.fm/reference/get_account_transactions)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_account_transactions`][cyhole.solana_fm.interaction.SolanaFM._get_account_transactions].
        """
        return await self._interaction._get_account_transactions(False, account, params)

    async def get_account_transactions_fees(self, account: str, dt_from: datetime | None = None, dt_to: datetime | None = None) -> GetAccountTransactionsFeesResponse:
        """
            Call the SolanaFM's API endpoint GET **[Account Transactions Fees](https://docs.solana.fm/reference/get_account_tx_fees)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_account_transactions_fees`][cyhole.solana_fm.interaction.SolanaFM._get_account_transactions_fees].
        """
        return await self._interaction._get_account_transactions_fees(False, account, dt_from, dt_to)

    async def get_account_transfers(self, account: str, params: GetAccountTransfersParam = GetAccountTransfersParam()) -> GetAccountTransfersResponse:
        """
            Call the SolanaFM's API endpoint GET **[Account Transfers](https://docs.solana.fm/reference/get_account_transfers_v1)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_account_transfers`][cyhole.solana_fm.interaction.SolanaFM._get_account_transfers].
        """
        return await self._interaction._get_account_transfers(False, account, params)

    async def get_account_transfers_csv_export(self, account: str, params: GetAccountTransfersCsvExportParam = GetAccountTransfersCsvExportParam()) -> GetAccountTransfersCsvExportResponse:
        """
            Call the SolanaFM's API endpoint GET **[Account Transfers CSV Export](https://docs.solana.fm/reference/download_csv_v1)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_account_transfers_csv_export`][cyhole.solana_fm.interaction.SolanaFM._get_account_transfers_csv_export].
        """
        return await self._interaction._get_account_transfers_csv_export(False, account, params)

    async def get_blocks(
        self,
        from_block: int | None = None,
        page_size: int = 50,
        page_type: str = SolanaFMBlocksPaginationType.BLOCK_NUMBER.value,
        ascending: bool | None = None
    ) -> GetBlocksResponse:
        """
            Call the SolanaFM's API endpoint GET **[Blocks](https://docs.solana.fm/reference/get_blocks_by_pagination)** for synchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_blocks`][cyhole.solana_fm.interaction.SolanaFM._get_blocks].
        """
        return await self._interaction._get_blocks(False, from_block, page_size, page_type, ascending)

    async def get_block(self, block_number: int) -> GetBlockResponse:
        """
            Call the SolanaFM's API endpoint GET **[Block](https://docs.solana.fm/reference/get_block)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_block`][cyhole.solana_fm.interaction.SolanaFM._get_block].
        """
        return await self._interaction._get_block(False, block_number)

    async def post_multiple_blocks(self, block_numbers: list[int], producer_details: bool = True) -> PostMultipleBlocksResponse:
        """
            Call the SolanaFM's API endpoint POST **[Multiple Blocks](https://docs.solana.fm/reference/get_multiple_blocks)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_multiple_blocks`][cyhole.solana_fm.interaction.SolanaFM._post_multiple_blocks].
        """
        return await self._interaction._post_multiple_blocks(False, block_numbers, producer_details)

    async def get_solana_daily_transaction_fees(self, dt: datetime = datetime.now()) -> GetSolanaDailyTransactionFeesResponse:
        """
            Call the SolanaFM's API endpoint GET **[Solana Daily Transaction Fees](https://docs.solana.fm/reference/get_daily_tx_fees)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_solana_daily_transaction_fees`][cyhole.solana_fm.interaction.SolanaFM._get_solana_daily_transaction_fees].
        """
        return await self._interaction._get_solana_daily_transaction_fees(False, dt)

    async def get_tagged_tokens_list(self) -> GetTaggedTokensListResponse:
        """
            Call the SolanaFM's API endpoint GET **[Tagged Tokens List](https://docs.solana.fm/reference/get_tokens_by_pagination)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_tagged_tokens_list`][cyhole.solana_fm.interaction.SolanaFM._get_tagged_tokens_list].
        """
        return await self._interaction._get_tagged_tokens_list(False)

    async def get_token_info_v0(self, address: str) -> GetTokenInfoV0Response:
        """
            Call the SolanaFM's API endpoint GET **[Token Info V0](https://docs.solana.fm/reference/get_token_by_account_hash)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_token_info_v0`][cyhole.solana_fm.interaction.SolanaFM._get_token_info_v0].
        """
        return await self._interaction._get_token_info_v0(False, address)

    async def post_token_multiple_info_v0(self, addresses: list[str]) -> PostTokenMultipleInfoV0Response:
        """
            Call the SolanaFM's API endpoint POST **[Token Multiple Info V0](https://docs.solana.fm/reference/get_tokens_by_account_hashes)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_token_multiple_info_v0`][cyhole.solana_fm.interaction.SolanaFM._post_token_multiple_info_v0].
        """
        return await self._interaction._post_token_multiple_info_v0(False, addresses)

    async def get_token_info_v1(self, address: str) -> GetTokenInfoV1Response:
        """
            Call the SolanaFM's API endpoint GET **[Token Info V1](https://docs.solana.fm/reference/get_one_token)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_token_info_v1`][cyhole.solana_fm.interaction.SolanaFM._get_token_info_v1].
        """
        return await self._interaction._get_token_info_v1(False, address)

    async def post_token_multiple_info_v1(self, addresses: list[str]) -> PostTokenMultipleInfoV1Response:
        """
            Call the SolanaFM's API endpoint POST **[Token Multiple Info V1](https://docs.solana.fm/reference/retrieve_multiple_tokens)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_token_multiple_info_v1`][cyhole.solana_fm.interaction.SolanaFM._post_token_multiple_info_v1].
        """
        return await self._interaction._post_token_multiple_info_v1(False, addresses)

    async def post_user_token_accounts(self, address: str, include_sol_balance: bool = False, tokens: list[str] | None = None) -> PostUserTokenAccountsResponse:
        """
            Call the SolanaFM's API endpoint POST **[User Token Accounts](https://docs.solana.fm/reference/get_user_token_accounts)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_user_token_accounts`][cyhole.solana_fm.interaction.SolanaFM._post_user_token_accounts].
        """
        return await self._interaction._post_user_token_accounts(False, address, include_sol_balance, tokens)

    async def get_mint_token_accounts(self, address: str, page: int | None = None, page_size: int | None = None) -> GetMintTokenAccountsResponse:
        """
            Call the SolanaFM's API endpoint GET **[Mint Token Accounts](https://docs.solana.fm/reference/get_token_accounts_for_token_mint)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_mint_token_accounts`][cyhole.solana_fm.interaction.SolanaFM._get_mint_token_accounts].
        """
        return await self._interaction._get_mint_token_accounts(False, address, page, page_size)

    async def get_on_chain_token_data(self, address: str) -> GetOnChainTokenDataResponse:
        """
            Call the SolanaFM's API endpoint GET **[On Chain Token Data](https://docs.solana.fm/reference/get_tfi_token_data)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_on_chain_token_data`][cyhole.solana_fm.interaction.SolanaFM._get_on_chain_token_data].
        """
        return await self._interaction._get_on_chain_token_data(False, address)

    async def get_token_supply(self, address: str) -> GetTokenSupplyResponse:
        """
            Call the SolanaFM's API endpoint GET **[Token Supply](https://docs.solana.fm/reference/get_token_circulating_supply)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_token_supply`][cyhole.solana_fm.interaction.SolanaFM._get_token_supply].
        """
        return await self._interaction._get_token_supply(False, address)

    async def get_transfer_transactions(self, transaction: str) -> GetTransferTransactionsResponse:
        """
            Call the SolanaFM's API endpoint GET **[Transfer Transactions](https://docs.solana.fm/reference/get_transfers)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_transfer_transactions`][cyhole.solana_fm.interaction.SolanaFM._get_transfer_transactions].
        """
        return await self._interaction._get_transfer_transactions(False, transaction)

    async def post_multiple_transfer_transactions(self, transactions: list[str]) -> PostMultipleTransferTransactionsResponse:
        """
            Call the SolanaFM's API endpoint POST **[Multiple Transfer Transactions](https://docs.solana.fm/reference/post_transfers)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._post_multiple_transfer_transactions`][cyhole.solana_fm.interaction.SolanaFM._post_multiple_transfer_transactions].
        """
        return await self._interaction._post_multiple_transfer_transactions(False, transactions)

    async def get_all_transfer_actions(self) -> GetAllTransferActionsResponse:
        """
            Call the SolanaFM's API endpoint GET **[All Transfer Actions](https://docs.solana.fm/reference/get_actions)** for asynchronous logic. 
            All the API endopint details are available on [`SolanaFM._get_all_transfer_actions`][cyhole.solana_fm.interaction.SolanaFM._get_all_transfer_actions].
        """
        return await self._interaction._get_all_transfer_actions(False)


from ..core.param import CyholeParam

class SolanaFMBlocksPaginationType(CyholeParam):
    """
        Pagination type for Solana FM blocks.
    """
    BLOCK_NUMBER = "blockNumber"
    BLOCK_TIME = "blockTime"